@@ -1,8 +1,6 @@
 import { onDocumentReady } from '../helpers'
-import { fixBlockquotes } from '../content'
 import { initialize as initMakeup } from '../makeup'
 
 onDocumentReady(() => {
   initMakeup()
-  fixBlockquotes()
 })
