@@ -80,17 +80,9 @@ defmodule ExDoc.Autolink do
     app_url(base_url, module, config, path, config.ext, "#{anchor}")
   end
 
-  defp string_app_module_url(string, tool, module, anchor, config) do
+  defp string_app_module_url(tool, module, anchor, config) do
     if Enum.any?(config.filtered_modules, &(&1.module == module)) do
-      # TODO: Remove on Elixir v1.14
-      prefix =
-        if unquote(Version.match?(System.version(), ">= 1.14.0")) do
-          ""
-        else
-          ~s|"#{string}" |
-        end
-
-      warn(config, prefix <> "reference to a filtered module")
+      warn(config, "reference to a filtered module")
       nil
     else
       app_module_url(tool, module, anchor, config)
@@ -269,7 +261,7 @@ defmodule ExDoc.Autolink do
 
     case {mode, Refs.get_visibility(ref)} do
       {_link_type, visibility} when visibility in [:public, :limited] ->
-        string_app_module_url(string, tool(module, config), module, anchor, config)
+        string_app_module_url(tool(module, config), module, anchor, config)
 
       {:regular_link, :undefined} ->
         nil
@@ -335,8 +327,8 @@ defmodule ExDoc.Autolink do
   end
 
   defp parse_url(string, mode, config) do
-    case Regex.run(~r{^(.+)/(\d+)$}, string) do
-      [_, left, right] ->
+    case Regex.run(~r{^(.+)/(\d+)(#.*)?$}, string) do
+      [_, left, right | maybe_fragment] ->
         with {:ok, arity} <- parse_arity(right) do
           {kind, rest} = kind(left)
 
@@ -344,11 +336,13 @@ defmodule ExDoc.Autolink do
             {:local, function} ->
               kind
               |> local_url(function, arity, config, string, mode: mode)
+              |> maybe_append_nested_fragment(maybe_fragment)
               |> maybe_remove_link(mode)
 
             {:remote, module, function} ->
               {kind, module, function, arity}
               |> remote_url(config, string, mode: mode)
+              |> maybe_append_nested_fragment(maybe_fragment)
               |> maybe_remove_link(mode)
 
             :error ->
@@ -491,7 +485,7 @@ defmodule ExDoc.Autolink do
         if same_module? do
           fragment(kind, name, arity)
         else
-          url = string_app_module_url(original_text, tool, module, nil, config)
+          url = string_app_module_url(tool, module, nil, config)
           url && url <> fragment(kind, name, arity)
         end
 
@@ -509,6 +503,18 @@ defmodule ExDoc.Autolink do
              (config.language != ExDoc.Language.Erlang or kind == :function) ->
         nil
 
+      {:regular_link, :hidden, :hidden}
+      when not same_module? ->
+        if warn? do
+          maybe_warn(config, ref, :hidden, %{
+            original_text: original_text,
+            module_visibility: :hidden,
+            same_module?: false
+          })
+        end
+
+        nil
+
       {_mode, _module_visibility, visibility} ->
         if warn? do
           maybe_warn(config, ref, visibility, %{original_text: original_text})
@@ -583,6 +589,19 @@ defmodule ExDoc.Autolink do
     warn(config, message)
   end
 
+  defp warn(
+         config,
+         {:type, module, _name, _arity},
+         :hidden,
+         %{original_text: original_text, module_visibility: :hidden, same_module?: false}
+       ) do
+    message =
+      "documentation references type \"#{original_text}\" but the module " <>
+        "#{inspect(module)} is #{format_visibility(:hidden, :module)}"
+
+    warn(config, message)
+  end
+
   defp warn(
          config,
          {kind, _module, _name, _arity},
@@ -611,6 +630,10 @@ defmodule ExDoc.Autolink do
   def format_visibility(:undefined, _kind), do: "undefined or private"
   def format_visibility(visibility, _kind), do: "#{visibility}"
 
+  defp maybe_append_nested_fragment(nil, _), do: nil
+  defp maybe_append_nested_fragment(url, []), do: url
+  defp maybe_append_nested_fragment(url, ["#" <> fragment]), do: url <> "-" <> fragment
+
   defp append_fragment(url, nil), do: url
   defp append_fragment(url, fragment), do: url <> "#" <> fragment
 end
