@@ -47,7 +47,8 @@ defmodule ExDoc.Formatter.HTML do
         generate_search(nodes_map, config) ++
         generate_not_found(nodes_map, config) ++
         generate_list(nodes_map.modules, nodes_map, config) ++
-        generate_list(nodes_map.tasks, nodes_map, config) ++ generate_index(config)
+        generate_list(nodes_map.tasks, nodes_map, config) ++
+        generate_redirects(config, ".html")
 
     generate_build(Enum.sort(all_files), build)
     config.output |> Path.join("index.html") |> Path.relative_to_cwd()
@@ -187,13 +188,6 @@ defmodule ExDoc.Formatter.HTML do
     File.write!(build, entries)
   end
 
-  defp generate_index(config) do
-    index_file = "index.html"
-    main_file = "#{config.main}.html"
-    generate_redirect(index_file, config, main_file)
-    [index_file]
-  end
-
   defp generate_not_found(nodes_map, config) do
     filename = "404.html"
     config = set_canonical_url(config, filename)
@@ -312,7 +306,7 @@ defmodule ExDoc.Formatter.HTML do
 
         is_binary(dir_or_files) and File.dir?(dir_or_files) ->
           dir_or_files
-          |> File.cp_r!(target_dir)
+          |> File.cp_r!(target_dir, dereference_symlinks: true)
           |> Enum.map(&Path.relative_to(&1, output))
 
         is_binary(dir_or_files) ->
@@ -390,6 +384,25 @@ defmodule ExDoc.Formatter.HTML do
     |> Enum.sort_by(fn extra -> GroupMatcher.group_index(groups, extra.group) end)
   end
 
+  def generate_redirects(config, ext) do
+    config.redirects
+    |> Map.new()
+    |> Map.put_new("index", config.main)
+    |> Enum.map(fn {from, to} ->
+      unless is_binary(from),
+        do: raise("expected a string for the source of a redirect, got: #{inspect(from)}")
+
+      unless is_binary(to),
+        do: raise("expected a string for the destination of a redirect, got: #{inspect(to)}")
+
+      source = from <> ext
+      destination = to <> ext
+      generate_redirect(source, config, destination)
+
+      source
+    end)
+  end
+
   defp disambiguate_id(extra, discriminator) do
     Map.put(extra, :id, "#{extra.id}-#{discriminator}")
   end
