@@ -21,7 +21,7 @@ defmodule ExDoc.Language.Erlang do
               nesting_info: nil,
               private: %{abst_code: any, callbacks: map, optional_callbacks: any, specs: map},
               title: binary,
-              type: :behaviour | :module
+              type: :module | :behaviour
             }
   def module_data(module, docs_chunk, _config) do
     if abst_code = Source.get_abstract_code(module) do
@@ -62,8 +62,8 @@ defmodule ExDoc.Language.Erlang do
   def function_data(entry, module_data) do
     {{kind, name, arity}, _anno, _signature, doc_content, metadata} = entry
 
-    # TODO: Edoc on Erlang/OTP24.1+ includes private functions in
-    # the chunk, so we manually yank them out for now.
+    # Edoc on Erlang/OTP24.1+ includes private functions in
+    # the chunk, so we manually yank them out.
     if kind == :function and doc_content != :hidden and
          function_exported?(module_data.module, name, arity) do
       function_data(name, arity, doc_content, module_data, metadata)
@@ -72,6 +72,33 @@ defmodule ExDoc.Language.Erlang do
     end
   end
 
+  defp function_data(name, arity, _doc_content, module_data, metadata) do
+    specs =
+      case Map.fetch(module_data.private.specs, {name, arity}) do
+        {:ok, spec} ->
+          [spec]
+
+        :error ->
+          case Map.fetch(module_data.private.specs, {module_data.module, name, arity}) do
+            {:ok, spec} ->
+              [spec]
+
+            :error ->
+              []
+          end
+      end
+
+    {file, line} = Source.fetch_function_location!(module_data, {name, arity})
+
+    %{
+      doc_fallback: fn -> equiv_data(module_data.module, file, line, metadata) end,
+      extra_annotations: [],
+      source_line: line,
+      source_file: file,
+      specs: specs
+    }
+  end
+
   defp equiv_data(module, file, line, metadata, prefix \\ "") do
     case metadata[:equiv] do
       nil ->
@@ -102,33 +129,6 @@ defmodule ExDoc.Language.Erlang do
     end
   end
 
-  defp function_data(name, arity, _doc_content, module_data, metadata) do
-    specs =
-      case Map.fetch(module_data.private.specs, {name, arity}) do
-        {:ok, spec} ->
-          [spec]
-
-        :error ->
-          case Map.fetch(module_data.private.specs, {module_data.module, name, arity}) do
-            {:ok, spec} ->
-              [spec]
-
-            :error ->
-              []
-          end
-      end
-
-    {file, line} = Source.fetch_function_location!(module_data, {name, arity})
-
-    %{
-      doc_fallback: fn -> equiv_data(module_data.module, file, line, metadata) end,
-      extra_annotations: [],
-      source_line: line,
-      source_file: file,
-      specs: specs
-    }
-  end
-
   @impl true
   def callback_data(entry, module_data) do
     {{_kind, name, arity}, anno, signature, _doc, metadata} = entry
@@ -239,6 +239,10 @@ defmodule ExDoc.Language.Erlang do
         {:attribute, anno, :type, ast} ->
           {name, _, _} = ast
           {name, anno, Code.Typespec.type_to_quoted(ast)}
+
+        {:attribute, anno, :nominal, ast} ->
+          {name, _, _} = ast
+          {name, anno, Code.Typespec.type_to_quoted(ast)}
       end
 
     formatted = format_spec(ast)
@@ -538,8 +542,8 @@ defmodule ExDoc.Language.Erlang do
             {name, _, _}, acc when name in [:<<>>, :..] ->
               {nil, acc}
 
-            # -1
-            {:-, _, [int]}, acc when is_integer(int) ->
+            # -1, +1
+            {op, _, [int]}, acc when is_integer(int) and op in [:+, :-] ->
               {nil, acc}
 
             # fun() (spec_to_quoted expands it to (... -> any())
