@@ -4,6 +4,7 @@ defmodule ExDoc.Retriever.ErlangTest do
   import TestHelper
 
   @moduletag :tmp_dir
+  @nominal_type if System.otp_release() >= "28", do: :nominal, else: :type
 
   describe "docs_from_modules/2" do
     @describetag :otp_eep59
@@ -274,23 +275,26 @@ defmodule ExDoc.Retriever.ErlangTest do
     end
 
     test "types", c do
-      erlc(c, :mod, ~S"""
+      erlc(c, :mod, """
       -module(mod).
-      -export_type([type1/0, equiv_type1/0, opaque1/0]).
+      -export_type([type1/0, equiv_type1/0, opaque1/0, nominal1/0]).
 
       -doc("type1/0 docs.").
       -type type1() :: atom().
 
-      -doc #{ equiv => type1/1 }.
+      -doc \#{equiv => type1/1}.
       -type equiv_type1() :: atom().
 
       -doc("opaque1/0 docs.").
       -opaque opaque1() :: atom().
+
+      -doc("nominal1/0 docs.").
+      -#{@nominal_type} nominal1() :: atom().
       """)
 
       config = %ExDoc.Config{source_url_pattern: "%{path}:%{line}"}
       {[mod], []} = Retriever.docs_from_modules([:mod], config)
-      [equiv_type1, opaque1, type1] = mod.typespecs
+      [equiv_type1, nominal1, opaque1, type1] = mod.typespecs
 
       assert opaque1.id == "t:opaque1/0"
       assert opaque1.type == :opaque
@@ -301,6 +305,15 @@ defmodule ExDoc.Retriever.ErlangTest do
       assert opaque1.spec |> Erlang.autolink_spec(current_kfa: {:type, :opaque1, 0}) ==
                "opaque1()"
 
+      assert nominal1.id == "t:nominal1/0"
+      assert nominal1.type == @nominal_type
+      assert nominal1.group == :Types
+      assert nominal1.signature == "nominal1()"
+      assert nominal1.doc |> DocAST.to_string() =~ "nominal1/0 docs."
+
+      assert nominal1.spec |> Erlang.autolink_spec(current_kfa: {:type, :nominal1, 0}) ==
+               "nominal1() :: <a href=\"https://www.erlang.org/doc/apps/erts/erlang.html#t:atom/0\">atom</a>()."
+
       assert type1.id == "t:type1/0"
       assert type1.type == :type
       assert type1.group == :Types
@@ -363,7 +376,7 @@ defmodule ExDoc.Retriever.ErlangTest do
       %% @doc
       %% mod docs.
       -module(mod).
-      -export([function1/0, function2/0]).
+      -export([function1/0, function2/0, hidden_function/0]).
 
       %% @doc
       %% function1/0 docs.
@@ -373,6 +386,13 @@ defmodule ExDoc.Retriever.ErlangTest do
       %% @doc
       %% function2/0 docs.
       function2() -> ok.
+
+      %% @doc hidden function docs.
+      %% @private
+      hidden_function() -> local_function().
+
+      %% @doc hidden function docs.
+      local_function() -> local_function().
       """)
 
       {[mod], []} = Retriever.docs_from_modules([:mod], %ExDoc.Config{})
@@ -474,21 +494,24 @@ defmodule ExDoc.Retriever.ErlangTest do
     end
 
     test "types", c do
-      erlc(c, :mod, ~S"""
+      erlc(c, :mod, """
       %% @doc Docs.
       -module(mod).
-      -export_type([type1/0, opaque1/0]).
+      -export_type([type1/0, opaque1/0, nominal1/0]).
 
       -type type1() :: atom().
       %% type1/0 docs.
 
       -opaque opaque1() :: atom().
       %% opaque1/0 docs.
+
+      -#{@nominal_type} nominal1() :: atom().
+      %% nominal1/0 docs.
       """)
 
       config = %ExDoc.Config{source_url_pattern: "%{path}:%{line}"}
       {[mod], []} = Retriever.docs_from_modules([:mod], config)
-      [opaque1, type1] = mod.typespecs
+      [nominal1, opaque1, type1] = mod.typespecs
 
       assert opaque1.id == "t:opaque1/0"
       assert opaque1.type == :opaque
@@ -498,6 +521,15 @@ defmodule ExDoc.Retriever.ErlangTest do
       assert opaque1.spec |> Erlang.autolink_spec(current_kfa: {:type, :opaque1, 0}) ==
                "opaque1()"
 
+      assert nominal1.id == "t:nominal1/0"
+      assert nominal1.type == @nominal_type
+      assert nominal1.group == :Types
+      assert nominal1.signature == "nominal1/0"
+      assert nominal1.doc |> DocAST.to_string() =~ "nominal1/0 docs."
+
+      assert nominal1.spec |> Erlang.autolink_spec(current_kfa: {:type, :nominal1, 0}) ==
+               "nominal1() :: <a href=\"https://www.erlang.org/doc/apps/erts/erlang.html#t:atom/0\">atom</a>()."
+
       assert type1.id == "t:type1/0"
       assert type1.type == :type
       assert type1.signature == "type1/0"
